package fr.takima.lotka_volterra.common;

import fr.takima.lotka_volterra.validator.NotNullValidator;

import java.math.BigDecimal;
import java.util.function.Function;

public class Vector {

    private static VectorComponent getVectorComponent(Angle angle, Distance distance, Function<Integer, Double> function) {
        var value = distance.value().multiply(BigDecimal.valueOf(function.apply(angle.value())));
        return new VectorComponent(value.intValue());
    }

    private static VectorComponent getVectorX(Angle angle, Distance distance) {
        return getVectorComponent(angle, distance, Math::sin);
    }

    private static VectorComponent getVectorY(Angle angle, Distance distance) {
        return getVectorComponent(angle, distance, Math::cos);
    }

    public static Vector of(Angle angle, Distance distance) {
        var deltaX = getVectorX(angle, distance);
        var deltaY = getVectorY(angle, distance);
        return new Vector(deltaX, deltaY);
    }

    private final VectorComponent x;
    private final VectorComponent y;

    public Vector(VectorComponent x, VectorComponent y) {
        new NotNullValidator("x", x).validate();
        new NotNullValidator("y", y).validate();
        this.x = x;
        this.y = y;
    }

    public VectorComponent x() {
        return x;
    }

    public VectorComponent y() {
        return y;
    }

}
