package fr.takima.lotka_volterra.moving;

import fr.takima.lotka_volterra.common.Angle;
import fr.takima.lotka_volterra.common.Distance;
import fr.takima.lotka_volterra.common.Vector;
import fr.takima.lotka_volterra.validator.NotNullValidator;

import java.util.HashSet;
import java.util.Set;

public final class Moving<T extends MovingEntity> {

    private final MovingConfigurer configurer;
    private final Set<T> entities;
    private final PositionUpdate<T> positionUpdate;

    public Moving(MovingConfigurer configurer, Set<T> entities, PositionUpdate<T> positionUpdate) {
        new NotNullValidator("configurer", configurer).validate();
        new NotNullValidator("entities", entities).validate();
        new NotNullValidator("positionUpdate", positionUpdate).validate();
        this.configurer = configurer;
        this.entities = entities;
        this.positionUpdate = positionUpdate;
    }

    public Set<T> run() {
        var next = new HashSet<T>();

        for (var entity : entities) {
            var randomVector = getRandomVector();
            var newPosition = entity.position().apply(randomVector);
            var updated = positionUpdate.apply(entity, newPosition);
            next.add(updated);
        }

        return next;
    }

    private Vector getRandomVector() {
        return Vector.of(getRandomAngle(), getRandomDistance());
    }

    private Angle getRandomAngle() {
        var rand = configurer.random().nextInt(Angle.MAX_VALUE);
        return new Angle(rand);
    }

    private Distance getRandomDistance() {
        var rand = configurer.random().nextInt(configurer.maxDistance().value().intValue());
        return new Distance(rand);
    }

}
