package fr.takima.lotka_volterra.moving;

import fr.takima.lotka_volterra.common.Distance;
import fr.takima.lotka_volterra.common.Vector;
import fr.takima.lotka_volterra.validator.NotNullValidator;

import java.math.BigDecimal;
import java.math.MathContext;

import static java.lang.Math.pow;

public record Position(PositionComponent x, PositionComponent y) {

    public Position {
        new NotNullValidator("x", x).validate();
        new NotNullValidator("y", y).validate();
    }

    public Position apply(Vector vector) {
        return new Position(x.apply(vector.x()), y.apply(vector.y()));
    }

    public Distance distanceTo(Position other) {
        var xPart = pow(x.value() - other.x.value(), 2);
        var yPart = pow(y.value() - other.y.value(), 2);
        var distanceValue = new BigDecimal(xPart + yPart).sqrt(MathContext.DECIMAL32);
        return new Distance(distanceValue);
    }

}
