package fr.takima.lotka_volterra.moving;

import fr.takima.lotka_volterra.common.VectorComponent;
import fr.takima.lotka_volterra.common.value.IntValue;
import fr.takima.lotka_volterra.validator.GreaterThanValidator;
import fr.takima.lotka_volterra.validator.LowerThanValidator;

import static java.lang.Math.max;
import static java.lang.Math.min;


public final class PositionComponent extends IntValue {

    private final Integer minValue;
    private final Integer maxValue;

    public PositionComponent(Integer value, Integer minValue, Integer maxValue) {
        super(value);
        new GreaterThanValidator<>("value", value, minValue).validate();
        new LowerThanValidator<>("value", value, maxValue).validate();
        this.minValue = minValue;
        this.maxValue = maxValue;
    }

    public PositionComponent apply(VectorComponent vectorComponent) {
        var sum = value + vectorComponent.value();
        var newValue = max(0, min(sum, maxValue));
        return new PositionComponent(newValue, minValue, maxValue);
    }

    @Override
    public String toString() {
        return "PositionComponent{" +
                "value=" + value +
                '}';
    }

}
