package fr.takima.lotka_volterra.predator;

import fr.takima.lotka_volterra.mating.MatingEntity;
import fr.takima.lotka_volterra.moving.Position;
import fr.takima.lotka_volterra.validator.NotNullValidator;

import java.util.Objects;

public record Predator(PredatorId id, Satiety satiety, Position position) implements MatingEntity<Predator> {

    public Predator {
        new NotNullValidator("id", id).validate();
        new NotNullValidator("satiety", satiety).validate();
        new NotNullValidator("position", position).validate();
    }

    public Predator withSatietyGain(Satiety gain) {
        return new Predator(id, satiety.add(gain), position);
    }

    public Predator withSatietyLoss(Satiety loss) {
        return new Predator(id, satiety.subtract(loss), position);
    }

    public boolean starvedToDeath() {
        return satiety.equals(Satiety.MIN);
    }

    public Predator withPosition(Position newPosition) {
        return new Predator(id, satiety, newPosition);
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        var predator = (Predator) o;
        return id.equals(predator.id);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id);
    }

}
