package fr.takima.lotka_volterra.predator;

import fr.takima.lotka_volterra.mating.ChildMaking;
import fr.takima.lotka_volterra.mating.Couple;
import fr.takima.lotka_volterra.validator.NotNullValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.Set;

@Component
public final class PredatorChildMaking implements ChildMaking<Predator> {

    private static final Logger log = LoggerFactory.getLogger(PredatorChildMaking.class);

    private final PredatorIdGenerator idGenerator;

    public PredatorChildMaking(PredatorIdGenerator idGenerator) {
        new NotNullValidator("idGenerator", idGenerator).validate();
        this.idGenerator = idGenerator;
    }

    @Override
    public Set<Predator> run(Couple<Predator> couple) {
        var child = new Predator(idGenerator.generate(), Satiety.MAX, couple.mate1().position());
        var children = Set.of(child);
        log.info("predators '{}' and '{}' made children", couple.mate1().id().value(), couple.mate2().id().value());
        return children;
    }

}
