package fr.takima.lotka_volterra.predator;

import fr.takima.lotka_volterra.common.value.IntValue;
import fr.takima.lotka_volterra.validator.GreaterThanValidator;
import fr.takima.lotka_volterra.validator.LowerThanValidator;

import static java.lang.Math.max;
import static java.lang.Math.min;

public final class Satiety extends IntValue {

    private static final int MIN_VALUE = 0;
    private static final int MAX_VALUE = 100;
    public static final Satiety MIN = new Satiety(MIN_VALUE);
    public static final Satiety MAX = new Satiety(MAX_VALUE);

    public Satiety(Integer value) {
        super(value);
        new GreaterThanValidator<>("value", value, MIN_VALUE).validate();
        new LowerThanValidator<>("value", value, MAX_VALUE).validate();
    }

    public Satiety add(Satiety satiety) {
        var sumValue = min(value + satiety.value, MAX_VALUE);
        return new Satiety(sumValue);
    }

    public Satiety subtract(Satiety satiety) {
        var differenceValue = max(value - satiety.value, MIN_VALUE);
        return new Satiety(differenceValue);
    }

    @Override
    public String toString() {
        return "Satiety{" +
                "value=" + value +
                '}';
    }

}
