package fr.takima.lotka_volterra.prey;

import fr.takima.lotka_volterra.mating.ChildMaking;
import fr.takima.lotka_volterra.mating.Couple;
import fr.takima.lotka_volterra.validator.NotNullValidator;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.util.Set;

@Component
public final class PreyChildMaking implements ChildMaking<Prey> {

    private static final Logger log = LoggerFactory.getLogger(PreyChildMaking.class);

    private final PreyIdGenerator idGenerator;

    public PreyChildMaking(PreyIdGenerator idGenerator) {
        new NotNullValidator("idGenerator", idGenerator).validate();
        this.idGenerator = idGenerator;
    }

    @Override
    public Set<Prey> run(Couple<Prey> couple) {
        var position = couple.mate1().position();
        var child1 = new Prey(idGenerator.generate(), position);
        var child2 = new Prey(idGenerator.generate(), position);
        var child3 = new Prey(idGenerator.generate(), position);
        var children = Set.of(child1, child2, child3);
        log.info("preys '{}' and '{}' made children", couple.mate1().id().value(), couple.mate2().id().value());
        return children;
    }

}
