package fr.takima.lotka_volterra.simulation;

import fr.takima.lotka_volterra.common.value.PositiveInt;
import fr.takima.lotka_volterra.mating.DefaultPredatorMatingConfigurer;
import fr.takima.lotka_volterra.mating.DefaultPreyMatingConfigurer;
import fr.takima.lotka_volterra.mating.MatingConfigurer;
import fr.takima.lotka_volterra.moving.*;
import fr.takima.lotka_volterra.predation.DefaultPredationConfigurer;
import fr.takima.lotka_volterra.predation.PredationConfigurer;
import fr.takima.lotka_volterra.predator.PredatorChildMaking;
import fr.takima.lotka_volterra.prey.PreyChildMaking;
import org.springframework.stereotype.Component;

@Component
public class DefaultSimulationConfigurer implements SimulationConfigurer {

    private final PositiveInt initialPreyCount;
    private final PositiveInt initialPredatorCount;
    private final PositiveInt stepCount;
    private final MapConfigurer map;
    private final PreyChildMaking preyChildMaking;
    private final PredatorChildMaking predatorChildMaking;
    private final MatingConfigurer preyMating;
    private final MatingConfigurer predatorMating;
    private final MovingConfigurer preyMoving;
    private final MovingConfigurer predatorMoving;
    private final PredationConfigurer predation;

    public DefaultSimulationConfigurer(SimulationProperties properties,
                                       DefaultMapConfigurer map,
                                       PreyChildMaking preyChildMaking,
                                       PredatorChildMaking predatorChildMaking,
                                       DefaultPreyMatingConfigurer preyMating,
                                       DefaultPredatorMatingConfigurer predatorMating,
                                       DefaultPreyMovingConfigurer preyMoving,
                                       DefaultPredatorMovingConfigurer predatorMoving,
                                       DefaultPredationConfigurer predation) {
        this.initialPreyCount = new PositiveInt(properties.getInitialPreyCount());
        this.initialPredatorCount = new PositiveInt(properties.getInitialPredatorCount());
        this.stepCount = new PositiveInt(properties.getStepCount());
        this.map = map;
        this.preyChildMaking = preyChildMaking;
        this.predatorChildMaking = predatorChildMaking;
        this.preyMating = preyMating;
        this.predatorMating = predatorMating;
        this.preyMoving = preyMoving;
        this.predatorMoving = predatorMoving;
        this.predation = predation;
    }

    @Override
    public PositiveInt initialPreyCount() {
        return initialPreyCount;
    }

    @Override
    public PositiveInt initialPredatorCount() {
        return initialPredatorCount;
    }

    @Override
    public PositiveInt stepCount() {
        return stepCount;
    }

    @Override
    public PreyChildMaking preyChildMaking() {
        return preyChildMaking;
    }

    @Override
    public PredatorChildMaking predatorChildMaking() {
        return predatorChildMaking;
    }

    @Override
    public MatingConfigurer preyMating() {
        return preyMating;
    }

    @Override
    public MatingConfigurer predatorMating() {
        return predatorMating;
    }

    @Override
    public MovingConfigurer preyMoving() {
        return preyMoving;
    }

    @Override
    public MovingConfigurer predatorMoving() {
        return predatorMoving;
    }

    @Override
    public PredationConfigurer predation() {
        return predation;
    }

    @Override
    public MapConfigurer map() {
        return map;
    }

}
