package fr.takima.lotka_volterra.simulation;

import fr.takima.lotka_volterra.predator.PredatorPositionUpdate;
import fr.takima.lotka_volterra.prey.PreyPositionUpdate;
import fr.takima.lotka_volterra.mating.Mating;
import fr.takima.lotka_volterra.moving.Moving;
import fr.takima.lotka_volterra.predation.Predation;
import fr.takima.lotka_volterra.simulation.step.Steps;
import fr.takima.lotka_volterra.validator.NotNullValidator;

public record Simulation(SimulationConfigurer configurer, Steps steps) {

    public Simulation {
        new NotNullValidator("configurer", configurer).validate();
        new NotNullValidator("steps", steps).validate();
    }

    public Simulation runNextStep() {
        var lastStep = steps.last();
        var initialPreys = lastStep.preys();
        var initialPredators = lastStep.predators();

        var predationResult = new Predation(configurer.predation(), initialPreys, initialPredators).run();
        var preyMatingResult = new Mating<>(configurer.preyMating(), predationResult.preys(), configurer().preyChildMaking()).run();
        var predatorMatingResult = new Mating<>(configurer.predatorMating(), predationResult.predators(), configurer().predatorChildMaking()).run();
        var preyMovingResult = new Moving<>(configurer.preyMoving(), preyMatingResult, new PreyPositionUpdate()).run();
        var predatorMovingResult = new Moving<>(configurer.predatorMoving(), predatorMatingResult, new PredatorPositionUpdate()).run();
        return new Simulation(configurer, steps.add(preyMovingResult, predatorMovingResult));
    }

}
