package fr.takima.lotka_volterra.simulation.step;

import fr.takima.lotka_volterra.common.value.PositiveInt;
import fr.takima.lotka_volterra.predator.Predator;
import fr.takima.lotka_volterra.prey.Prey;
import fr.takima.lotka_volterra.simulation.Populations;
import fr.takima.lotka_volterra.validator.NotNullValidator;

import java.util.Set;

public record Step(Number number, Set<Prey> preys, Set<Predator> predators) {

    public static final class Number extends PositiveInt {

        public Number(Integer value) {
            super(value);
        }

        public Number increment() {
            return new Number(value + 1);
        }

    }

    public Step {
        new NotNullValidator("number", number).validate();
        new NotNullValidator("preys", preys).validate();
        new NotNullValidator("predators", predators).validate();
    }

    public Populations populations() {
        var preysPopulation = new PositiveInt(preys.size());
        var predatorsPopulation = new PositiveInt(predators.size());
        return new Populations(preysPopulation, predatorsPopulation);
    }

}
