package fr.takima.lotka_volterra.simulation.step;

import fr.takima.lotka_volterra.predator.Predator;
import fr.takima.lotka_volterra.prey.Prey;
import fr.takima.lotka_volterra.validator.NotNullValidator;

import java.util.*;

import static java.util.stream.IntStream.range;

public final class Steps {

    private final List<Step> items;

    public Steps(Collection<Step> items) {
        new NotNullValidator("items", items).validate();
        this.items = new ArrayList<>();
        range(0, items.size())
                .forEach(i -> addStepToItems(items, new Step.Number(i)));
    }

    public int size() {
        return items.size();
    }

    public Step last() {
        return items.get(size() - 1);
    }

    public Steps add(Set<Prey> preys, Set<Predator> predators) {
        var nextNumber = last().number().increment();
        var step = new Step(nextNumber, preys, predators);
        var newSteps = new ArrayList<>(items);
        newSteps.add(step);
        return new Steps(newSteps);
    }

    private void addStepToItems(Collection<Step> items, Step.Number stepNumber) {
        get(items, stepNumber).ifPresentOrElse(
                this.items::add,
                () -> {
                    var message = "could not find step " + stepNumber.value();
                    throw new IllegalArgumentException(message);
                });
    }

    private static Optional<Step> get(Collection<Step> items, Step.Number stepNumber) {
        return items.stream()
                .filter(it -> it.number().equals(stepNumber))
                .findAny();
    }

}
