package fr.takima.lotka_volterra.moving;


import fr.takima.lotka_volterra.common.Distance;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import java.math.BigDecimal;
import java.math.MathContext;

import static org.assertj.core.api.Assertions.assertThat;

class MovingEntityTest {

    private static final Position POSITION_0_0 = new Position(new PositionComponent(0, -100, 100), new PositionComponent(0, -100, 100));
    private static final Position POSITION_1_1 = new Position(new PositionComponent(1, -100, 100), new PositionComponent(1, -100, 100));
    private static final FakeMovingEntity ENTITY_1 = new FakeMovingEntity(POSITION_0_0);
    private static final FakeMovingEntity ENTITY_2 = new FakeMovingEntity(POSITION_1_1);

    @Nested
    class DistanceToTest {

        @Test
        void ok() {
            var actual = ENTITY_1.distanceTo(ENTITY_2);
            assertThat(actual).isEqualTo(new Distance(new BigDecimal(2).sqrt(MathContext.DECIMAL32)));
        }

    }

    @Nested
    class IsWithinTest {

        @Test
        void _true() {
            var actual = ENTITY_1.isWithin(new Distance(2), ENTITY_2);
            assertThat(actual).isTrue();
        }

        @Test
        void _false() {
            var actual = ENTITY_1.isWithin(new Distance(1), ENTITY_2);
            assertThat(actual).isFalse();
        }

    }

    record FakeMovingEntity(Position position) implements MovingEntity {

    }

}
