package fr.takima.lotka_volterra.predator;

import fr.takima.lotka_volterra.mating.Couple;
import fr.takima.lotka_volterra.moving.Position;
import fr.takima.lotka_volterra.moving.PositionComponent;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.doReturn;

@ExtendWith(MockitoExtension.class)
class PredatorChildMakingTest {

    private static final Position PARENT_1_POSITION = new Position(new PositionComponent(0, -100, 100), new PositionComponent(0, -100, 100));
    private static final Position PARENT_2_POSITION = new Position(new PositionComponent(1, -100, 100), new PositionComponent(1, -100, 100));
    private static final Predator PARENT_1 = new Predator(
            new PredatorId("1"),
            Satiety.MAX,
            PARENT_1_POSITION);
    private static final Predator PARENT_2 = new Predator(
            new PredatorId("2"),
            Satiety.MAX,
            PARENT_2_POSITION);
    private static final PredatorId CHILD_ID = new PredatorId("3");
    private static final Predator CHILD = new Predator(
            CHILD_ID,
            Satiety.MAX,
            PARENT_1_POSITION);
    private static final Couple<Predator> COUPLE = new Couple<>(PARENT_1, PARENT_2);

    @Mock
    private PredatorIdGenerator idGenerator;

    @InjectMocks
    private PredatorChildMaking predatorChildMaking;

    @Test
    void ok() {
        doReturn(CHILD_ID)
                .when(idGenerator).generate();

        var actual = predatorChildMaking.run(COUPLE);
        assertThat(actual).usingRecursiveFieldByFieldElementComparator()
                .containsExactlyInAnyOrder(CHILD);
    }

}
