package fr.takima.lotka_volterra.prey;

import fr.takima.lotka_volterra.mating.Couple;
import fr.takima.lotka_volterra.moving.Position;
import fr.takima.lotka_volterra.moving.PositionComponent;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;

import static org.assertj.core.api.Assertions.assertThat;
import static org.mockito.Mockito.doReturn;

@ExtendWith(MockitoExtension.class)
class PreyChildMakingTest {

    private static final Position PARENT_1_POSITION = new Position(new PositionComponent(0, -100, 100), new PositionComponent(0, -100, 100));
    private static final Position PARENT_2_POSITION = new Position(new PositionComponent(1, -100, 100), new PositionComponent(1, -100, 100));
    private static final Prey PARENT_1 = new Prey(new PreyId("1"), PARENT_1_POSITION);
    private static final Prey PARENT_2 = new Prey(new PreyId("2"), PARENT_2_POSITION);
    private static final PreyId CHILD_1_ID = new PreyId("3");
    private static final Prey CHILD_1 = new Prey(CHILD_1_ID, PARENT_1_POSITION);
    private static final PreyId CHILD_2_ID = new PreyId("4");
    private static final Prey CHILD_2 = new Prey(CHILD_2_ID, PARENT_1_POSITION);
    private static final PreyId CHILD_3_ID = new PreyId("5");
    private static final Prey CHILD_3 = new Prey(CHILD_3_ID, PARENT_1_POSITION);
    private static final Couple<Prey> COUPLE = new Couple<>(PARENT_1, PARENT_2);

    @Mock
    private PreyIdGenerator idGenerator;

    @InjectMocks
    private PreyChildMaking preyChildMaking;

    @Test
    void ok() {
        doReturn(CHILD_1_ID, CHILD_2_ID, CHILD_3_ID)
                .when(idGenerator).generate();

        var actual = preyChildMaking.run(COUPLE);
        assertThat(actual).usingRecursiveFieldByFieldElementComparator()
                .containsExactlyInAnyOrder(CHILD_1, CHILD_2, CHILD_3);
    }

}
