package fr.takima.lotka_volterra.prey;

import fr.takima.lotka_volterra.moving.Position;
import fr.takima.lotka_volterra.moving.PositionComponent;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import static org.assertj.core.api.Assertions.assertThat;

class PreyTest {

    private static final Position POSITION_0_0 = new Position(new PositionComponent(0, -100, 100), new PositionComponent(0, -100, 100));
    private static final Position POSITION_1_1 = new Position(new PositionComponent(0, -100, 100), new PositionComponent(0, -100, 100));
    private static final Prey ENTITY_1 = new Prey(new PreyId("1"), POSITION_0_0);
    private static final Prey ENTITY_1_WITH_POSITION = new Prey(new PreyId("1"), POSITION_1_1);
    private static final Prey ENTITY_2 = new Prey(new PreyId("2"), POSITION_1_1);

    @Nested
    class WithPositionTest {

        @Test
        void ok() {
            var actual = ENTITY_1.withPosition(POSITION_1_1);
            assertThat(actual).isEqualTo(ENTITY_1_WITH_POSITION);
        }

    }

    @Nested
    class EqualsTest {

        @Test
        void with_itself_returns_true() {
            var actual = ENTITY_1.equals(ENTITY_1);
            assertThat(actual).isTrue();
        }

        @Test
        void with_null_returns_false() {
            var actual = ENTITY_1.equals(null);
            assertThat(actual).isFalse();
        }

        @Test
        void with_other_class_returns_null() {
            var actual = ENTITY_1.equals("");
            assertThat(actual).isFalse();
        }

        @Test
        void with_other_id_returns_false() {
            var actual = ENTITY_1.equals(ENTITY_2);
            assertThat(actual).isFalse();
        }

        @Test
        void with_same_id_returns_true() {
            var actual = ENTITY_1.equals(ENTITY_1_WITH_POSITION);
            assertThat(actual).isTrue();
        }

    }

}
